#
# inst.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'rbconfig'
require 'ftools'


class Installer < File

  include Config

  BINDIR = CONFIG['bindir']
  RBDIR = [ CONFIG['libdir'],
            CONFIG['ruby_install_name'],
            CONFIG['MAJOR'] + '.' + CONFIG['MINOR'],
            'site_ruby' ].join( '/' )
  SODIR = [ RBDIR, CONFIG['arch'] ].join( '/' )

  class << self

    def execute
      if not ARGV[0] then
        unless com_setup then
          puts 'setup failed'
          exit 1
        end

        puts 'setup ok, entering installation step ...'
        unless com_install then
          puts 'install failed'
          exit 1
        end

        puts 'done installation.'

      elsif ARGV[0] == 'setup'   then com_setup
      elsif ARGV[0] == 'install' then com_install
      else
        puts 'usage: ruby ./setup.rb [setup|install]'
        exit 1
      end
    end


    def newdir( dir )
      mkpath dir
      dir
    end

    def chdir( dir )
      cur = Dir.pwd
      Dir.chdir dir
      yield
      Dir.chdir cur
    end

    def lib_install( libname )
      unless File.directory? libname then
        # probabry being called from external ...
        return
      end

      chdir( libname ) do
        dn = LIBDIR + '/' + libname
        mkpath dn
        yield dn
      end
    end

    def install_rb( to )
      Dir.foreach( '.' ) do |fn|
        if /\.rb\z/o === fn and file? fn then
          install fn, to, 644, true
        end
      end
    end

    def make
      system 'make'
    end

    def install_so( to )
      fnames = nil
      Dir.open( '.' ) {|d| fnames = d.to_a }
      ['so', 'sl', 'dll', 'o'].each do |ext|
        exp = /\.#{ext}\z/
        fn = fnames.find {|f| exp === f }
        if fn then
          install fn, to, 555, true
          break
        end
      end
    end

    def make_install_so( to )
      load './extconf' unless exist? 'Makefile'
      make
      install_so to
    end

    def so_install( libname )
      unless File.directory? libname then
        # probabry being called from external ...
        return
      end

      chdir( libname ) do
        dn = SOLIB + '/' + libname
        mkpath dn
        yield dn
      end
    end

    def install_all( to )
      Dir.foreach( '.' ) do |fn|
        if file? fn then
          install fn, to, 644, true
        end
      end
    end

  end

end
